#include "ListModel.h"

#define MAX_ITEMS 10

ListModel::ListModel(QWidget *parent) : QAbstractListModel(parent) {
    youtubeSearch = new YouTubeSearch();
    connect(youtubeSearch, SIGNAL(gotVideo(Video*)), this, SLOT(addVideo(Video*)));
    connect(youtubeSearch, SIGNAL(finished(int)), this, SLOT(searchFinished(int)));
    searching = false;
    canSearchMore = true;
}

ListModel::~ListModel() {
    delete youtubeSearch;
}

int ListModel::rowCount(const QModelIndex &/*parent*/) const {
    int count = videos.size();

    // add the message item
    if (videos.isEmpty() || !searching)
        count++;

    return count;
}

QVariant ListModel::data(const QModelIndex &index, int role) const {

    int row = index.row();

    if (row == videos.size()) {

        QPalette palette;
        QFont boldFont;
        boldFont.setBold(true);

        switch (role) {
        case ItemTypeRole:
            return ItemTypeShowMore;
        case Qt::DisplayRole:
        case Qt::StatusTipRole:
            if (searching) return "Searching...";
            if (videos.isEmpty()) return "No videos";
            if (canSearchMore) return "Show " + QString::number(MAX_ITEMS) + " More";
            else return "No more videos";
        case Qt::TextAlignmentRole:
            return QVariant(int(Qt::AlignHCenter | Qt::AlignVCenter));
        case Qt::ForegroundRole:
            return palette.color(QPalette::Dark);
        case Qt::FontRole:
            return boldFont;
        default:
            return QVariant();
        }

    } else if (row < 0 || row >= videos.size())
        return QVariant();

    Video *video = videos.at(row);

    switch (role) {
    case ItemTypeRole:
        return ItemTypeVideo;
    case VideoRole:
        return QVariant::fromValue(QPointer<Video>(video));
    case ActiveTrackRole:
        return row == m_activeRow;
    case Qt::DisplayRole:
    case Qt::StatusTipRole:
        return video->title();
        /*
        case Qt::ToolTipRole:

            QString tooltip;
            if (!element.firstChildElement().text().isEmpty()) {
                tooltip.append(QString("<b>").append(element.firstChildElement().text()).append("</b><br/>"));
            }
            if (!fromDate.isEmpty()) {
                tooltip.append("<i>Pubblicato il</i> ").append(fromDate);
            }
            if (!toDate.isEmpty()) {
                tooltip.append("<br/><i>Scadenza</i>: ").append(toDate);
            }
            tooltip.append("<br/><i>Tipo</i>: ").append(typeName)
                .append("<br/><i>Id</i>: ").appen    QFont boldFont;
    boldFont.setBold(true);d(id);
            return tooltip;
            */

         case StreamUrlRole:
        return video->streamUrl();
    }

    return QVariant();
}

void ListModel::setActiveRow( int row) {
    if ( rowExists( row ) ) {

        m_activeRow = row;

        // setStateOfRow( row, Item::Played );

        int oldactiverow = m_activeRow;

        if ( rowExists( oldactiverow ) )
            emit dataChanged( createIndex( oldactiverow, 0 ), createIndex( oldactiverow, columnCount() - 1 ) );

        emit dataChanged( createIndex( m_activeRow, 0 ), createIndex( m_activeRow, columnCount() - 1 ) );
        emit activeRowChanged(row);

    } else {
        m_activeRow = -1;
        // emit activeTrackChanged( 0 );
    }
    // emit activeRowChanged( m_activeRow );
}

int ListModel::nextRow() const {
    int nextRow = m_activeRow + 1;
    if (rowExists(nextRow))
        return nextRow;
    return -1;
}

Video* ListModel::videoAt( int row ) const {
    if ( rowExists( row ) )
        return videos.at( row );
    return 0;
}

Video* ListModel::activeVideo() const {
    return videoAt(m_activeRow);
}

void ListModel::search(QString query) {
    while (!videos.isEmpty())
        delete videos.takeFirst();
    reset();
    this->query = query;
    youtubeSearch->search(query, MAX_ITEMS, 1);
    searching = true;
}

void ListModel::searchMore(int max) {
    if (searching) return;
    youtubeSearch->search(query, max, videos.size()+1);
    searching = true;
}

void ListModel::searchMore() {
    searchMore(MAX_ITEMS);
}

void ListModel::searchNeeded() {
    int remainingRows = videos.size() - m_activeRow;
    int rowsNeeded = MAX_ITEMS - remainingRows;
    if (rowsNeeded > 0)
    searchMore(rowsNeeded);
}

void ListModel::abortSearch() {
    while (!videos.isEmpty())
        delete videos.takeFirst();
    reset();
    youtubeSearch->abort();
    searching = false;
}

void ListModel::searchFinished(int total) {
    searching = false;
    canSearchMore = total > 0;
    reset();
}

void ListModel::addVideo(Video* video) {

    videos << video;

    // QModelIndex startIndex = createIndex(videos.size()-2, 0);
    // QModelIndex endIndex = createIndex(videos.size()-2, 0);

    // autoplay
    if (videos.size() == 1) {
        setActiveRow(0);
    }

    // emit rowsInserted(startIndex, videos.size() - 1, videos.size());
    // FIXME
    // emit dataChanged( createIndex( videos.size()-1, 0 ), createIndex( videos.size()-1, columnCount() - 1 ) );
    emit reset();

}
