#ifndef LISTMODEL_H
#define LISTMODEL_H

#include "video.h"
#include "youtubesearch.h"

enum DataRoles {
    ItemTypeRole = Qt::UserRole,
    VideoRole,
    StreamUrlRole,
    ActiveTrackRole
};

enum ItemTypes {
    ItemTypeVideo = 1,
    ItemTypeShowMore
};

class ListModel : public QAbstractListModel {

    Q_OBJECT

public:

    ListModel(QWidget *parent);
    ~ListModel();

    // inherited from QAbstractListModel
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    // int rowCount( const QModelIndex& parent = QModelIndex() ) const { Q_UNUSED( parent ); return videos.size(); }
    int columnCount( const QModelIndex& parent = QModelIndex() ) const { Q_UNUSED( parent ); return 4; }
    QVariant data(const QModelIndex &index, int role) const;

    // custom methods
    void setActiveRow( int row );
    bool rowExists( int row ) const { return (( row >= 0 ) && ( row < videos.size() ) ); }
    int activeRow() const { return m_activeRow; } // returns -1 if there is no active row
    int nextRow() const;

    Video* videoAt( int row ) const;
    Video* activeVideo() const;

    // video search methods
    void search(QString query);
    void abortSearch();

public slots:
    void searchMore();
    void searchNeeded();
    void addVideo(Video* video);
    void searchFinished(int total);

signals:
    void activeRowChanged(int);

private:
    void searchMore(int max);

    YouTubeSearch *youtubeSearch;
    QString query;
    bool searching;
    bool canSearchMore;

    QList<Video*> videos;

    // the row being played
    int m_activeRow;

};

#endif
