#include <QtNetwork>
#include "MediaView.h"
#include "playlist/PrettyItemDelegate.h"
#include "networkaccess.h"

MediaView::MediaView(QWidget *parent) : QWidget(parent) {

    QBoxLayout *layout = new QHBoxLayout();
    layout->setMargin(0);

    QSplitter *splitter = new QSplitter(this);
    splitter->setChildrenCollapsible(false);

    listView = new QListView(this);
    listView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
    listView->setFrameShape( QFrame::NoFrame );
    listView->setItemDelegate(new Playlist::PrettyItemDelegate(this));
    listView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    /* TODO dragndrop
    listView->setDragEnabled(true);
    listView->setAcceptDrops(true);
    listView->setDropIndicatorShown(true);
    listView->setDragDropMode(QAbstractItemView::InternalMove);
    */
    listView->setMinimumSize(320,240);
    connect(listView, SIGNAL(activated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));

    listModel = new ListModel(this);
    connect(listModel, SIGNAL(activeRowChanged(int)), this, SLOT(activeRowChanged(int)));
    listView->setModel(listModel);

    splitter->addWidget(listView);

    videoWidget = new Phonon::VideoWidget(this);
    videoWidget->setMinimumSize(320,240);
    splitter->addWidget(videoWidget);

    layout->addWidget(splitter);
    setLayout(layout);
}

MediaView::~MediaView() {

}

void MediaView::setMediaObject(Phonon::MediaObject *mediaObject) {
    this->mediaObject = mediaObject;
    Phonon::createPath(this->mediaObject, videoWidget);
    connect(mediaObject, SIGNAL(aboutToFinish()), this, SLOT(aboutToFinish()));
    connect(mediaObject, SIGNAL(finished()), this, SLOT(skip()));
    connect(mediaObject, SIGNAL(stateChanged(Phonon::State, Phonon::State)),
            this, SLOT(stateChanged(Phonon::State, Phonon::State)));
    connect(mediaObject, SIGNAL(currentSourceChanged(Phonon::MediaSource)),
            this, SLOT(currentSourceChanged(Phonon::MediaSource)));

}

void MediaView::setQuery(QString query) {
    this->query = query;
    listModel->search(query);
}

void MediaView::disappear() {

}

void MediaView::stateChanged(Phonon::State newState, Phonon::State /* oldState */)
{
    switch (newState) {

         case Phonon::ErrorState:
        qDebug() << "Phonon error:" << mediaObject->errorString() << mediaObject->errorType();
        break;

         case Phonon::PlayingState:
        qDebug("playing");
        break;

         case Phonon::StoppedState:
        qDebug("stopped");
        // qDebug() << "hasVideo" << mediaObject->hasVideo();
        mediaObject->play();
        break;

         case Phonon::PausedState:
        qDebug("paused");
        break;

         case Phonon::BufferingState:
        qDebug("buffering");
        break;

         case Phonon::LoadingState:
        qDebug("loading");
        break;

         default:
        ;
    }
}

void MediaView::pause() {
    switch( mediaObject->state() ) {
    case Phonon::PlayingState:
        mediaObject->pause();
        break;
    default:
        mediaObject->play();
        break;
    }
}

void MediaView::fullscreen() {
    videoWidget->setFullScreen(!videoWidget->isFullScreen());
}

void MediaView::stop() {
    listModel->abortSearch();
}

void MediaView::activeRowChanged(int row) {
    Video *video = listModel->videoAt(row);
    if (!video) return;
    QUrl streamUrl = video->streamUrl();
    qDebug() << "setCurrentSource" << streamUrl.toString();

    /*
    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
    QNetworkReply* reply = manager->get(QNetworkRequest(streamUrl));
    sleep(10);
    Phonon::MediaSource source(reply);
    */
    QNetworkAccessManager *manager = The::networkAccessManager();
    connect(manager, SIGNAL(finished(QNetworkReply*)),
         this, SLOT(replyFinished(QNetworkReply*)));

    // play will be called on StoppedState
    // when the media is loaded
    // Phonon::MediaSource mediaSource(streamUrl);
    mediaObject->setCurrentSource(streamUrl);

    // ensure active item is visible
    QModelIndex index = listModel->index(row, 0, QModelIndex());
    listView->scrollTo(index, QAbstractItemView::EnsureVisible);

    // ensure we always have 10 videos ahead
    listModel->searchNeeded();
}

void MediaView::itemActivated(const QModelIndex &index) {
    if (listModel->rowExists(index.row()))
        listModel->setActiveRow(index.row());
    // the user doucleclicked on the "Search More" item
    else listModel->searchMore();
}

void MediaView::aboutToFinish() {
    /*
    int nextRow = listModel->nextRow();
    if (nextRow == -1) return;
    Video* video = listModel->videoAt(nextRow);
    QUrl streamUrl = video->streamUrl();
    qDebug() << "Enqueing" << streamUrl;
    mediaObject->enqueue(streamUrl);
    */
}

void MediaView::currentSourceChanged(const Phonon::MediaSource source) {
    qDebug() << "Source changed:" << source.url();
}

void MediaView::skip() {
    int nextRow = listModel->nextRow();
    if (nextRow == -1) return;
    listModel->setActiveRow(nextRow);
}

void MediaView::openWebPage() {
    Video* video = listModel->activeVideo();
    if (!video) return;
    mediaObject->pause();
    QDesktopServices::openUrl(video->webpage());
}

void MediaView::replyFinished(QNetworkReply* reply) {}
