#include "SearchView.h"

#include <QtGui>

SearchView::SearchView(QWidget *parent) : QWidget(parent) {

    // setBackgroundRole(QPalette::Base);
    // setAutoFillBackground(true);

    // setStyleSheet("background: url(:/images/app.png) no-repeat right bottom");
    // setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #f6f7fa, stop: 1 #dadbde);");
    QFont biggerFont;
    biggerFont.setPointSize(biggerFont.pointSize()*1.5);

    QFont smallerFont;
    smallerFont.setPointSize(smallerFont.pointSize()*.85);
    smallerFont.setBold(true);

    QVBoxLayout *layout = new QVBoxLayout();
    layout->setMargin(40);

    QLabel *welcomeLabel =
            new QLabel("<h1>Welcome to <a href='http://flavio.tordini.org/minitube'>Minitube</a>,</h1>", this);
    welcomeLabel->setOpenExternalLinks(true);
    // welcomeLabel->setPixmap(QPixmap(":/images/app.png").scaledToHeight(48, Qt::SmoothTransformation));
    // welcomeLabel->setForegroundRole(QPalette::Dark);
    layout->addWidget(welcomeLabel);

    layout->addSpacing(30);

    QLabel *tipLabel = new QLabel("Enter a keyword to start watching videos.", this);
    tipLabel->setFont(biggerFont);
    layout->addWidget(tipLabel);

    // layout->addSpacing(5);

    QHBoxLayout *searchLayout = new QHBoxLayout();

    queryEdit = new SearchLineEdit(this);
    queryEdit->setFont(biggerFont);
    queryEdit->setMaximumWidth(400);
    queryEdit->setFocus(Qt::OtherFocusReason);
    // queryEdit->setStyleSheet("background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 palette(midlight), stop: 1 palette(base));");
    // connect(queryEdit, SIGNAL(returnPressed()), this, SLOT(watch()));
    connect(queryEdit, SIGNAL(search(const QString&)), this, SLOT(watch(const QString&)));
    searchLayout->addWidget(queryEdit);

    searchLayout->addSpacing(10);

    QPushButton *watchButton = new QPushButton("Watch", this);
    watchButton->setDefault(true);
    // watchButton->setGeometry(
    // watchButton->setFont(biggerFont);
    // searchLayout->setStretchFactor(watchButton, 100);
    connect(watchButton, SIGNAL(clicked()), this, SLOT(watch()));
    searchLayout->addWidget(watchButton);

    searchLayout->addStretch();

    layout->addItem(searchLayout);

    layout->addSpacing(30);

    QHBoxLayout *otherLayout = new QHBoxLayout();

    recentKeywordsLayout = new QVBoxLayout();
    recentKeywordsLayout->setAlignment(Qt::AlignTop);
    QLabel *label = new QLabel(tr("Recent keywords").toUpper(), this);
    label->setForegroundRole(QPalette::Dark);
    label->setFont(smallerFont);
    recentKeywordsLayout->addWidget(label);

    otherLayout->addLayout(recentKeywordsLayout);

    layout->addLayout(otherLayout);

    layout->addStretch();

    setLayout(layout);

    // watchButton->setMinimumHeight(queryEdit->height());
}

void SearchView::updateRecentKeywords() {

    // cleanup
    QLayoutItem *item;
    while ((item = recentKeywordsLayout->takeAt(1)) != 0) {
        item->widget()->close();
        delete item;
    }


    QSettings settings;
    QStringList keywords = settings.value("recentKeywords").toStringList();
    foreach (QString keyword, keywords) {
        QLabel *itemLabel = new QLabel("<a href=\"" + keyword
                                       + "\" style=\"color:palette(text); text-decoration:none\">"
                                       + keyword + "</a>", this);
        connect(itemLabel, SIGNAL(linkActivated(QString)), this, SLOT(watch(QString)));
        recentKeywordsLayout->addWidget(itemLabel);
    }

}

void SearchView::watch() {
    QString query = queryEdit->text();
    if (queryEdit->text().length() == 0)
        queryEdit->setFocus(Qt::OtherFocusReason);
    else
        watch(query);
}

void SearchView::watch(QString query) {

    // save keyword
    QSettings settings;
    QStringList keywords = settings.value("recentKeywords").toStringList();
    keywords.removeAll(query);
    keywords.prepend(query);
    while (keywords.size() > 8)
        keywords.removeLast();
    settings.setValue("recentKeywords", keywords);

    emit search(query);
}
