#include "SettingsView.h"

SettingsView::SettingsView( QWidget *parent ) : QWidget(parent) {

    QSettings settings;

    QBoxLayout *layout = new QVBoxLayout(this);
    layout->setAlignment(Qt::AlignCenter);

    QLabel *heading = new QLabel(QString("<h1>").append(tr("Preferences")).append("</h1>"), this);
    layout->addWidget(heading);

    QGroupBox *groupBox = new QGroupBox();
    QGridLayout *gbLayout = new QGridLayout();
    QLabel *label = new QLabel("<h3>Event sources</h3>");
    gbLayout->addWidget(label);

    int size = settings.beginReadArray("urls");
    QList<QUrl> urls;
    for (int i = 0; i < size; ++i) {
        settings.setArrayIndex(i);
        QUrl url = settings.value("url").value<QUrl>();
        urls.append(url);
    }
    settings.endArray();
    
    urlEdit = new QLineEdit(settings.value("url").toUrl().toString());
    urlEdit->setFocus();
    connect(urlEdit, SIGNAL(textEdited(const QString&)), this, SLOT(validateUrl(const QString&)));
    label->setBuddy(urlEdit);
    gbLayout->addWidget(urlEdit);
    urlLabel = new QLabel("Example: http://www.mysite.com/monitaur.php");
    gbLayout->addWidget(urlLabel, 1, Qt::AlignTop);
    groupBox->setLayout(gbLayout);
    layout->addWidget(groupBox);

    QPushButton *backButton = new QPushButton("&Done");
    backButton->setDefault(true);
    connect(backButton, SIGNAL(clicked()), this, SLOT(storeSettings()));
    // connect(backButton, SIGNAL(clicked()), parent, SLOT(goBack()));
    layout->addWidget(backButton, 1, Qt::AlignBottom);

}

void SettingsView::storeSettings() {

    QList<QUrl> urls;

    QSettings settings;
    settings.beginWriteArray("urls");
    for (int i = 0; i < urls.size(); ++i) {
        QUrl url = urls.at(i);
            // validate url
    if (!validateUrl(urlEdit->text())) {
        urlEdit->selectAll();
        return;
    }
        
        settings.setArrayIndex(i);
        // settings.setValue("url", list.at(i).userName);
    }
    settings.endArray();

    settings.setValue("url", urlEdit->text().trimmed());
}

bool SettingsView::validateUrl(const QString &text) {
    QString urlText = text.trimmed();
    if (urlText.isEmpty()) {
        urlLabel->setText("Please type a URL");
        urlEdit->setFocus();
        return false;
    }
    QUrl url = QUrl(text.trimmed());
    if (url.isValid()) {
        urlLabel->setText("Ok this is a valid URL");
        return true;
    } else {
        urlLabel->setText("This is not a valid URL");
        urlEdit->setFocus();
        return false;
    }
}
