#include "youtubesearch.h"
#include "youtubestreamreader.h"

YouTubeSearch::YouTubeSearch() : QObject() {
    http = new QHttp();
    syncHttp = new SyncHttp(this);
}

void YouTubeSearch::request(QHttp *http, QUrl url) {
    QHttp::ConnectionMode mode = url.scheme().toLower() == "https" ?
                                 QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp;
    http->setHost(url.host(), mode, url.port() == -1 ? 0 : url.port());
    QHttpRequestHeader header("GET", url.path() + "?" + url.encodedQuery());
    header.setValue("Host", url.host());
    // header.setValue("User-Agent", "Minitube (http://flavio.tordini.org/minitube)");
    httpBuffer = new QBuffer();
    if (!httpBuffer->open(QBuffer::ReadWrite)) {
        qDebug("Cannot open buffer");
    }
    qDebug() << "GET " << url.toEncoded();
    httpRequestId = http->request(header, 0, httpBuffer);
}

void YouTubeSearch::search(QString query, int max, int skip) {
    this->abortFlag = false;
    http->disconnect(SIGNAL(requestFinished(int, bool)));
    connect(http, SIGNAL(requestFinished(int, bool)), this, SLOT(searchFinished(int, bool)));

    QString urlString = QString("http://gdata.youtube.com/feeds/api/videos?q=%1&max-results=%2&start-index=%3").arg(query, QString::number(max), QString::number(skip));
    QUrl url(urlString);
    request(http, url);
}

void YouTubeSearch::searchFinished(int requestId, bool error) {
    if (requestId != this->httpRequestId) return;

    if (abortFlag) {
        delete httpBuffer;
        return;
    }

    if (error) {
        qDebug(http->errorString().toUtf8());
    } else {
        parseResults(httpBuffer);
    }

    delete httpBuffer;
}

void YouTubeSearch::parseResults(QBuffer *buffer) {
    // qDebug() << buffer->data();
    YouTubeStreamReader reader;
    if (!reader.read(buffer)) {
        qDebug() << "Error while parsing XML";
    }
    videos = reader.getVideos();

    // and now for the dirty work...
    Video *video;
    foreach (video, videos) {
        if (abortFlag) return;

        // get the stream url
        this->getVideoUrl(video);

        if (abortFlag) return;

        // preload the thumb
        video->preloadThumbnail(syncHttp);

        // send it to the model
        emit gotVideo(video);
    }

    emit finished(videos.size());
}

bool YouTubeSearch::getVideoUrl(Video *video) {

    // https://develop.participatoryculture.org/trac/democracy/browser/trunk/tv/portable/flashscraper.py

    QUrl webpage = video->webpage();
    // if (webpage == ) return false;
    // qDebug() << webpage.toString();

    // Get Video ID
    // youtube-dl line 428
    // QRegExp re("^((?:http://)?(?:\\w+\\.)?youtube\\.com/(?:(?:v/)|(?:(?:watch(?:\\.php)?)?\\?(?:.+&)?v=)))?([0-9A-Za-z_-]+)(?(1).+)?$");
    QRegExp re("^http://www\\.youtube\\.com/watch\\?v=([0-9A-Za-z_-]+)$");
    bool match = re.exactMatch(webpage.toString());
    if (!match || re.numCaptures() < 1) return false;
    QString videoId = re.cap(1);
    // if (!videoId) return false;
    // qDebug() << videoId;

    // Get Video Token
    QUrl normalizedUrl = QUrl(QString("http://www.youtube.com/get_video_info?video_id=")
                              .append(videoId).append("&el=embedded&ps=default&eurl="));
    QString videoInfo = syncHttp->get(normalizedUrl);
    // qDebug() << videoInfo;
    re = QRegExp("^.*&token=([^&]+).*$");
    match = re.exactMatch(videoInfo);
    if (!match || re.numCaptures() < 1) return false;
    QString videoToken = re.cap(1);
    // FIXME proper decode
    videoToken = videoToken.replace("%3D", "=");
    // qDebug() << "token" << videoToken;

    QUrl videoUrl = QUrl(QString("http://www.youtube.com/get_video?video_id=")
                         .append(videoId)
                         .append("&t=").append(videoToken)
                         .append("&eurl=&el=embedded&ps=default&fmt=18"));

    // qDebug() << videoUrl;



    // follow redirects
    /*
    while (true) {
        QHttpResponseHeader headers = syncHttp->head(videoUrl);
        qDebug() << headers.values();
        if (headers.hasKey("Location")) {
            videoUrl = QUrl(headers.value("Location"), QUrl::StrictMode);
            // qDebug() << videoUrl;
        } else break;
    }*/

    video->setStreamUrl(videoUrl);

    return true;
}

QList<Video*> YouTubeSearch::getResults() {
    return videos;
}

void YouTubeSearch::abort() {
    this->abortFlag = true;
}
